// ************************************************************************************************ 
// PWM Generator fr Wechselrichter, 16KHz Pulsfrequenz / 50 Hz Ausgangsfrequenz
// ber den Sollwert an PortD kann die Hhe der Ausgangsspannung eingestellt werden
// An PortB1 und B2 wird jeweils ein IR2109 zur Ansteuerung einer Halbbrcke angeschlossen
// ************************************************************************************************ 
// Copyright 
// Version 
/*
* Chip type           : ATMEGA 88
* Clock frequency     : clock 20 Mhz 

Ein- und Ausgnge
PORT B 0 (14) =  LED nach Masse
PORT B 1 (15) = (OC1A/PCINT1) 	PWM Ausgabe, 
PORT B 2 (16) = (SS/OC1B/PCINT2) 	PWM Ausgabe, 
PORT B 3 (17) =  PD7 an Steuerung 
PORT B 4 (18) = 
PORT B 5 (19) = 
PORT B 6 ( 9) = 
PORT B 7 (10) = 

PORT C 0 (23) = PD2 an Steuerung 
PORT C 1 (24) = PD3 an Steuerung 
PORT C 2 (25) = PD4 an Steuerung 
PORT C 3 (26) = PD5 an Steuerung 
PORT C 4 (27) = PD6 an Steuerung 	
PORT C 5 (28) = PD7 an Steuerung

PORT D 0 ( 2) = 
PORT D 1 ( 3) = 
PORT D 2 ( 4) = INT 0 = PD0 an Steuerung und X3-3 
PORT D 3 ( 5) = INT 1 = PD1 an Steuerung und X3-2 
PORT D 4 ( 6) = Freigabe Halbbrcke 2
PORT D 5 (11) = 
PORT D 6 (12) = Freigabe Halbbrcke 1
PORT D 7 (13) = LED nach Masse
*/


// 
// 

// ************************************************************************************************ 
// Include Dateien laden
// ************************************************************************************************ 
#include	<avr\io.h>	//AVR Register und Konstantendefinitionen
#include	<avr\interrupt.h> //AVR Interrupt Vektoren
#include	<stdlib.h>
#include	<inttypes.h>
#include 	"wr_sinustabelle_01.h"

#define top  1249 	// Maximalwert ICR1 , ergibt 16KHz PWM Frequenz


// ************************************************************************************************ 
//                       Globale Variablen          
// ************************************************************************************************ 

volatile unsigned int zwischenspeicher_a=0;
volatile unsigned int zwischenspeicher_b=0;
volatile unsigned int omega=0;
              
// ************************************************************************************************ 
//                       Initialisierung          
// ************************************************************************************************ 
void initial(void)
{    
// **** Einrichten von Ein- und Ausgngen ******************************************************* 
DDRB = 0x07;      	// PortB1 und B2 Ausgnge 
DDRC = 0x00;      	// PortC Eingnge 
DDRD = 0x00;      	// PortD Eingnge 
PORTD= 0xff;      	// PortD Pull up aktiviert

// **** Einrichten von PWM Betrieb *************************************************************** 
// Der Timer 1 wird im Fast PWM Modus betrieben
// Mode 	WGM13	WGM12	WGM11	WGM10
// 14 		1 		1 		1 		0 		Fast PWM 		ICR1 		BOTTOM 	TOP

// fOCnxPWM = (fclk_I/O)/(N *(1 + TOP))	N=Prescaler
// fOCnxPWM = 20MHz / (1+1024) = 19.512,19Hz PWM Frequenz
// fOCnxPWM = 20MHz / (1+1249) = 16.000 Hz PWM Frequenz
// fOCnxPWM = 20MHz / (1+1665) = 12.004,8 Hz PWM Frequenz
// fOCnxPWM = 20MHz / (1+1999) = 10.000 Hz PWM Frequenz

// CS12 	CS11 	CS10 	Description
// 0 		0 		1 		clkI/O/1 (No prescaling)
// 0 		1 		0 		clkI/O/8 (From prescaler)
// 0 		1 		1 		clkI/O/64 (From prescaler)
// 1 		0 		0 		clkI/O/256 (From prescaler)
// 1 		0 		1 		clkI/O/1024 (From prescaler)

// COM1A1/COM1B1 	COM1A0/COM1B0 	Description
// 	1 			1		Set OC1A/OC1B on Compare Match, clearOC1A/OC1B at BOTTOM (inverting mode)

// COM1A1 	COM1A0 	COM1B1 	COM1B0	 -	 -	 WGM11 	WGM10 => Bits im Register TCCR1A
TCCR1A  = ((1 <<WGM11)|(1<<COM1A1)|(1<<COM1A0)|(1<<COM1B1)|(1<<COM1B0)) ; 	// Mode 14 + inverting mode
						
	
// ICNC1 	ICES1	 -	 WGM13 	WGM12 	CS12 		CS11 		CS10 => Bits im Register TCCR1B	
TCCR1B =((1 <<WGM13)|(1 <<WGM12)|(1<<CS10)) ; 	// Mode 14 + No prescaling


ICR1=top;	// 1249 ergibt 16000 KHz PWM Frequenz @ 20MHz fr Mode 14

// - 		- 	ICIE1 	- 	- 	OCIE1B 	OCIE1A 	TOIE1  	=> Bits im Register TIMSK1
TIMSK1= (1 <<TOIE1);	// Die ISR (TIMER1_OVF_vect) beim Erreichen von ICR1 aufrufen

TCNT1 = 0 ; 		// reset timer / counterregister

sei() ;

}
// ************************************************************************************************ 




// ******************************************************************* 
//                       Timerinterrupt 1  
// @ 16KHz ==> wird alle 62,5sec aufgerufen
// ******************************************************************* 
ISR (TIMER1_OVF_vect) // signal handler for tcnt1 overflow interrupt
{
						// 3,10sec Bearbeitungszeit @20MHz
OCR1A=zwischenspeicher_a;					
OCR1B=zwischenspeicher_b;
						
omega++; 
if(omega>=320) omega=0;

}



// ********************************************************************************** 
// Hauptprogramm
// ********************************************************************************** 



int main(void)
{

// ********************************************************************************** 
//                       Variablen  in main
// ********************************************************************************** 
unsigned char sollwert=0;	// Ausgangssollwert
unsigned long hilf=0; 		// Hilfsspeicher


// ******** Initialisierung **********************************
initial();	// Initialisierung Ports etc.
// ******** Hauptschleife ************************************	
    
  while(1)			// 4,50 sec Bearbeitungszeit @20MHz
  {

PORTB |= _BV(0);	// LED ein (nur um zu sehen, ob der Prozessor luft!)


PORTD |= _BV(6);	// Freigabe Halbbrcke 1
PORTD |= _BV(4);	// Freigabe Halbbrcke 2



sollwert= (PINC & 0x3f);	// Sollwert von 0 bis 63 entspricht 0 bis 100% (na ja, fast)


hilf = ((uint32_t)sinustabelle[omega]*(uint32_t)sollwert);
hilf=hilf >> 8;	// die Sinustabelle ist normiert auf 4*TOP-Wert 
			// daher wird hier durch 256 anstelle von durch 64 gerechnet
			// ist in der Verarbeitung schneller

if (omega<=160)
{
  zwischenspeicher_a=(top-hilf);
  zwischenspeicher_b=top;
}
else
{
  zwischenspeicher_b = (top-hilf);
  zwischenspeicher_a = top;
}

// ******************************************************** 
  } // Ende while(1)Endlosschleife
  return 0;
}// Ende Hauptprogramm
